clear
clc
sp_num=7;       % The number of spacecrafts
iteration_n=200;
error_gB_D=zeros(3,3,126,4);  
error_GB_D=zeros(3,3,3,126,4); 
t_step=1./2.^(0:5);
for jj=1:6
    step_length=0.08/2^(jj-1);
    xc_x=-5:step_length:5;                          % Trajectory of x component of the barycenter and y, z components are invariant constant.
    t=repmat((1:length(xc_x))',1,sp_num);    % Time series of the spacecrafts. Columns represent the spacecrafts.
    tc=mean(t,2);                          % Time series at the barycenter.
    t_num=length(tc);                      % The number of time series.
    % xc=[xc_x',zeros(t_num,1),zeros(t_num,1)];  % The Cartesian coordinates of the barycenter of the constellation in the Earth center reference frame.Rows represent the times.
    xc=[xc_x',zeros(t_num,1),5*ones(t_num,1)];
    [dB,ddB] = GetTheoryGradientsB(xc,'Dipole');  % Theory values of the linear and quadratic gradients of magnetic field.
    load x
    x=x/300;                                   % Regulate the size of the constellation.
    x=x-mean(x);                             % Ensure the barycenter at (0,0,0)
    x=repmat(permute(x,[3,2,1]),[t_num,1,1]);    % Rows represent the times. Columns represent the vector components. Pages represent the spacecrafts.
    x0=x+repmat(xc,[1,1,sp_num]);             % The Cartesian coordinates of the spacecrafts in the Earth center reference frame.
    B=zeros(t_num,3,sp_num);
    for ii=1:sp_num
%         B(:,:,ii)=GetTheoryB(x0(:,:,ii),'Dipole');
        B(:,:,ii)=GetTheoryB_Discretization_error(x0(:,:,ii),'Dipole',step_length); % Values of magnetic field with discretization_error.
    end
    [R2,gB,GB,error_gB,error_GB] = ALQG7(x,B,tc,iteration_n,dB,ddB);
    error_gB_D(:,:,:,jj)=error_gB(:,:,1:2^(jj-1):end,end);
    error_GB_D(:,:,:,:,jj)=error_GB(:,:,:,1:2^(jj-1):end,end);
end


error_gB11=reshape(error_gB_D(1,1,26,:),[],1);      %(-3,0,5)
error_gB22=reshape(error_gB_D(2,2,26,:),[],1);
error_gB13=reshape(error_gB_D(1,3,26,:),[],1);
error_gB33=reshape(error_gB_D(3,3,26,:),[],1);
error_GB111=reshape(error_GB_D(1,1,1,26,:),[],1);
error_GB131=reshape(error_GB_D(1,3,1,26,:),[],1);
error_GB221=reshape(error_GB_D(2,2,1,26,:),[],1);
error_GB331=reshape(error_GB_D(3,3,1,26,:),[],1);
error_GB122=reshape(error_GB_D(1,2,2,26,:),[],1);
error_GB232=reshape(error_GB_D(2,3,2,26,:),[],1);
error_GB113=reshape(error_GB_D(1,1,3,26,:),[],1);
error_GB133=reshape(error_GB_D(1,3,3,26,:),[],1);
error_GB223=reshape(error_GB_D(2,2,3,26,:),[],1);
error_GB333=reshape(error_GB_D(3,3,3,26,:),[],1);
figure('Color','w','units','pixels','position',[50,50,800,800],'paperunits','centimeters','papersize',[21,29.7])
panel_width=0.35;
panel_height=0.4;
panel_gap=0.07;
left=0.08;
bottom=0.08;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Define a CVD-friendly color order with ten distinct colors
cvdColorOrder = [
    230/255, 159/255, 0;  % Orange
    86/255, 180/255, 233/255; % Sky Blue
    0, 158/255, 115/255; % Bluish Green
    213/255, 94/255, 0; % Vermillion
    0, 114/255, 178/255; % Blue
    204/255, 121/255, 167/255; % Reddish Purple
    153/255, 153/255, 153/255; % Gray
    117/255, 112/255, 179/255; % Purplish Blue
    240/255, 228/255, 66/255; % Yellow
    0, 0, 0;         % Black
];
colororder(cvdColorOrder)     % Set the color order for the current figure
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
subplot('position',[left,bottom+(panel_height+panel_gap),panel_width,panel_height]);
error_gB_plot=[error_gB11,error_gB22,error_gB13,error_gB33];
plot(t_step,error_gB_plot,'LineWidth',1,'Marker','.')
legend('\partialBx/\partialx','\partialBy/\partialy','\partialBz/\partialx','\partialBz/\partialz','location','west')
xlabel('discretization step (s)')
ylabel('Errors (%)')
axis([0,1,0,0.05])
text(0.05,0.9,'barycenter at (-3, 0, 5)','Units','normalized')
text(0.01,0.975,'(a)','Units','normalized')

subplot('position',[left+(panel_width+panel_gap),bottom+(panel_height+panel_gap),panel_width,panel_height]);
error_GB_plot=[error_GB111,error_GB131,error_GB221,error_GB331,error_GB122,error_GB232,error_GB113,error_GB133,error_GB223,error_GB333];
plot(t_step,error_GB_plot,'LineWidth',1,'Marker','.')
legend('\partial^2Bx/\partialx^2','\partial^2Bx/\partialx\partialz','\partial^2Bx/\partialy^2',...
       '\partial^2Bx/\partialz^2','\partial^2By/\partialx\partialy','\partial^2By/\partialy\partialz',...
       '\partial^2Bz/\partialx^2','\partial^2Bz/\partialx\partialz','\partial^2Bz/\partialy^2','\partial^2Bz/\partialz^2',...
       'location','none','position',[0.86,0.70,0.132,0.25])
xlabel('discretization step (s)')
ylabel('Errors (%)')
axis([0,1,0,0.5])
text(0.05,0.9,'barycenter at (-3, 0, 5)','Units','normalized')
text(0.01,0.975,'(b)','Units','normalized')

error_gB11=reshape(error_gB_D(1,1,106,:),[],1);      %(3.4,0,5)
error_gB22=reshape(error_gB_D(2,2,106,:),[],1);
error_gB13=reshape(error_gB_D(1,3,106,:),[],1);
error_gB33=reshape(error_gB_D(3,3,106,:),[],1);
error_GB111=reshape(error_GB_D(1,1,1,106,:),[],1);
error_GB131=reshape(error_GB_D(1,3,1,106,:),[],1);
error_GB221=reshape(error_GB_D(2,2,1,106,:),[],1);
error_GB331=reshape(error_GB_D(3,3,1,106,:),[],1);
error_GB122=reshape(error_GB_D(1,2,2,106,:),[],1);
error_GB232=reshape(error_GB_D(2,3,2,106,:),[],1);
error_GB113=reshape(error_GB_D(1,1,3,106,:),[],1);
error_GB133=reshape(error_GB_D(1,3,3,106,:),[],1);
error_GB223=reshape(error_GB_D(2,2,3,106,:),[],1);
error_GB333=reshape(error_GB_D(3,3,3,106,:),[],1);

subplot('position',[left,bottom,panel_width,panel_height]);
error_gB_plot=[error_gB11,error_gB22,error_gB13,error_gB33];
plot(t_step,error_gB_plot,'LineWidth',1,'Marker','.')
legend('\partialBx/\partialx','\partialBy/\partialy','\partialBz/\partialx','\partialBz/\partialz','location','west')
xlabel('discretization step (s)')
ylabel('Errors (%)')
axis([0,1,0,0.05])
text(0.05,0.9,'barycenter at (3.4,0,5)','Units','normalized')
text(0.01,0.975,'(c)','Units','normalized')

subplot('position',[left+(panel_width+panel_gap),bottom,panel_width,panel_height]);
error_GB_plot=[error_GB111,error_GB131,error_GB221,error_GB331,error_GB122,error_GB232,error_GB113,error_GB133,error_GB223,error_GB333];
plot(t_step,error_GB_plot,'LineWidth',1,'Marker','.')
legend('\partial^2Bx/\partialx^2','\partial^2Bx/\partialx\partialz','\partial^2Bx/\partialy^2',...
       '\partial^2Bx/\partialz^2','\partial^2By/\partialx\partialy','\partial^2By/\partialy\partialz',...
       '\partial^2Bz/\partialx^2','\partial^2Bz/\partialx\partialz','\partial^2Bz/\partialy^2','\partial^2Bz/\partialz^2',...
       'location','none','position',[0.86,0.23,0.132,0.25])
xlabel('discretization step (s)')
ylabel('Errors (%)')
axis([0,1,0,0.5])
text(0.05,0.9,'barycenter at (3.4, 0, 5)','Units','normalized')
text(0.01,0.975,'(d)','Units','normalized')
print('discretization_error','-dpdf','-bestfit')
% close