function B = GetTheoryB_Discretization_error(xx,ModelOption,step_length)
% Return the theory values of magnetic field for different model with discretization error.
% input: x: The Cartesian coordinates of the spacecrafts in the Earth center reference frame.
%           x can be a row vector or a matrix.
%           If x is a matrix, columns represent the vector components and rows can represent times.       
%        ModelOption: 'Diople' or 'Fluxrope'
% output: B: The magnetic field that corresponds to x.
num=length(xx(:,1));
B=zeros(size(xx));
switch ModelOption
    case 'Dipole'
        Bx=@(x,y,z) -3*30008*x.*z./(x.^2+y.^2+z.^2).^2.5;
        By=@(x,y,z) -3*30008*y.*z./(x.^2+y.^2+z.^2).^2.5;
        Bz=@(x,y,z) 30008*(x.^2+y.^2-2*z.^2)./(x.^2+y.^2+z.^2).^2.5;
        for ii=1:num
            B(ii,1)=integral(@(x) Bx(x,xx(ii,2),xx(ii,3)),xx(ii,1)-step_length/2,xx(ii,1)+step_length/2)/step_length;
            B(ii,2)=integral(@(x) By(x,xx(ii,2),xx(ii,3)),xx(ii,1)-step_length/2,xx(ii,1)+step_length/2)/step_length;
            B(ii,3)=integral(@(x) Bz(x,xx(ii,2),xx(ii,3)),xx(ii,1)-step_length/2,xx(ii,1)+step_length/2)/step_length;
        end
%     case 'Fluxrope'
%         B=FluxropeB(x);
    otherwise
        B=[];
        warning('Unexpected magntic field model. The B is a empty array.')
end

